@@ -19,13 +19,18 @@
 #include <config.h>
 #endif
 
-#ifndef VERSION
-    #define VERSION "unknown"
-#endif
+#ifdef _WIN32
+#include <windows.h>
+#endif // _WIN32
 
 // OS includes
 #include <stdio.h>
+#ifndef _WIN32
 #include <unistd.h>
+#endif // _WIN32
+
+#include "../config-krename.h"
+
 // chmod:
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -120,14 +125,13 @@ void BatchRenamer::processFilenames()
     for( unsigned int i = 0; i < static_cast<unsigned int>(m_files->count()); i++) 
     {
         m_counter_index = 0;
-
         if( m_renameMode == eRenameMode_Rename ) // final Path = source Path
         {
             (*m_files)[i].setDstDirectory( (*m_files)[i].srcDirectory() );
-
+            
             KUrl url = (*m_files)[i].srcUrl();
             url.setFileName( QString::null );
-
+            
             (*m_files)[i].setDstUrl( url );
             
         } 
@@ -136,60 +140,45 @@ void BatchRenamer::processFilenames()
             (*m_files)[i].setDstUrl( m_destination );
             (*m_files)[i].setDstDirectory( m_destination.path() );
         }
-
+        
         if( i > 0 && m_reset )
             findCounterReset( i );
-
-	qDebug("SRCFILENAME       : %s", (*m_files)[i].srcFilename().toUtf8().data() );
-        qDebug("DSTFILENAME SHOULD: %s", processString( text, (*m_files)[i].srcFilename(), i ).toUtf8().data() );
+        
+        //qDebug("SRCFILENAME       : %s", (*m_files)[i].srcFilename().toUtf8().data() );
+        //qDebug("DSTFILENAME SHOULD: %s", processString( text, (*m_files)[i].srcFilename(), i ).toUtf8().data() );
         (*m_files)[i].setDstFilename( processString( text, (*m_files)[i].srcFilename(), i ) );
-        qDebug("DSTFILENAME IS    : %s", (*m_files)[i].dstFilename().toUtf8().data());
+        //qDebug("DSTFILENAME IS    : %s", (*m_files)[i].dstFilename().toUtf8().data());
         (*m_files)[i].setDstExtension( processString( extext, (*m_files)[i].srcExtension(), i ) );
-
-        (void)applyManualChanges( i );
-
-        // Assemble filenames
-        //parseSubdirs( &m_files[i] );
-        // TODO: DOM 
-        // ESCAPE HERE
-        /*
-        m_files[i].src.name = BatchRenamer::buildFilename( &m_files[i].src, true );
-
+        
         // Let's run the plugins that change the final filename,
         // i.e the encodingsplugin
-        m_files[i].dst.name = parsePlugins( i, m_files[i].dst.name, TYPE_FINAL_FILENAME );
-
-        m_files[i].dst.name = BatchRenamer::buildFilename( &m_files[i].dst, true );
-        */
-
+        int errors = 0;
+        QString name = executePlugin( i, (*m_files)[i].dstFilename(), ePluginType_Filename, errors, NULL );
+        if( !name.isNull() ) 
+            (*m_files)[i].setDstFilename( name );
+                
         /*
          * take care of renamed directories and
          * correct the paths of their contents
          */
-	if( (m_renameMode == eRenameMode_Rename ||
-	     m_renameMode == eRenameMode_Move) &&
-	    (*m_files)[i].isDirectory() )
-	{
-	    qDebug("Names=%s %s", (*m_files)[i].srcFilename().toUtf8().data(), (*m_files)[i].dstFilename().toUtf8().data() );
-	    const QString topDir  = (*m_files)[i].srcDirectory() + '/' + (*m_files)[i].srcFilename();
-	    const QString replace = (*m_files)[i].dstDirectory() + '/' + (*m_files)[i].dstFilename();
-	    qDebug("TOPFILE=%s", topDir.toUtf8().data() );
-	    qDebug("REPALCE=%s", replace.toUtf8().data() );
-
-	    for( int z = i + 1; z < m_files->count(); z++ ) 
-	    {
-		const QString & dir = (*m_files)[z].realSrcDirectory();
-		qDebug("DIR=%s\n", dir.toUtf8().data() );
-		qDebug("SRC=%s\n", topDir.toUtf8().data() );
-		if( dir.startsWith( topDir ) )
-		{
-		    QString newDir = replace + "/" + dir.right( dir.length() - topDir.length() );
-		    qDebug("newDir=%s", newDir.toUtf8().data() );
-		    (*m_files)[z].setOverrideSrcDirectory( newDir ); 
-		}
-	    }
-	}
-	    
+        if( (m_renameMode == eRenameMode_Rename ||
+             m_renameMode == eRenameMode_Move) &&
+            (*m_files)[i].isDirectory() )
+        {
+            const QString topDir  = (*m_files)[i].srcDirectory() + '/' + (*m_files)[i].srcFilename();
+            const QString replace = (*m_files)[i].dstDirectory() + '/' + (*m_files)[i].dstFilename();
+            
+            for( int z = i + 1; z < m_files->count(); z++ ) 
+            {
+                const QString & dir = (*m_files)[z].realSrcDirectory();
+                if( dir.startsWith( topDir ) )
+                {
+                    QString newDir = replace + "/" + dir.right( dir.length() - topDir.length() );
+                    (*m_files)[z].setOverrideSrcDirectory( newDir ); 
+                }
+            }
+        }
+        
 #if 0      
         if( m_files[i].dir && (m_mode == RENAME || m_mode == MOVE) ) {
             for( unsigned int c = i; c < m_files.count(); c++ ) {
@@ -244,9 +233,19 @@ void BatchRenamer::processFiles( ProgressDialog* p )
         if( p->wasCancelled() )
             break;
 
+        // Assemble filenames
+        createMissingSubDirs( (*m_files)[i], p );
+        
         KIO::JobFlags flags  = (m_overwrite ? KIO::Overwrite : KIO::DefaultFlags) | KIO::HideProgressInfo;
         KIO::Job*     job    = NULL;
         const KUrl &  srcUrl =  (*m_files)[i].srcUrl();
+        if( srcUrl == dstUrl ) 
+        {
+            p->error( i18n("Cannot rename: source and target filename are equal: %1", srcUrl.prettyUrl()) );
+            (*m_files)[i].setError( 1 );
+            errors++;
+            continue;
+        }
                     
         switch( m_renameMode ) 
         {
@@ -276,7 +275,7 @@ void BatchRenamer::processFiles( ProgressDialog* p )
 
         if( job && !NetAccess::synchronousRun( job, p ) ) 
         {
-            p->error( i18n("Error during renaming %1 (from %2)", dstUrl.prettyUrl(), srcUrl.prettyUrl()) );
+            p->error( i18n("Error renaming %2 (to %1)", dstUrl.prettyUrl(), srcUrl.prettyUrl()) );
             (*m_files)[i].setError( 1 );
             errors++;
         } 
@@ -301,7 +300,7 @@ void BatchRenamer::processFiles( ProgressDialog* p )
     p->print( i18n("KRename finished the renaming process."), "krename" );
     p->print( i18n("Press close to quit!") );
     bool enableUndo = (m_renameMode != eRenameMode_Copy);
-    p->renamingDone( enableUndo, this, errors );
+    p->renamingDone( true, enableUndo, this, errors );
     
 #if 0
     delete object;
@@ -334,8 +333,6 @@ void BatchRenamer::processFiles( ProgressDialog* p )
         if( !extext.isEmpty() )
             m_files[i].dst.extension = processString( extext, m_files[i].src.extension, i );
 
-        (void)applyManualChanges( i );
-
         // Assemble filenames
         parseSubdirs( &m_files[i] );
         // TODO: DOM 
@@ -423,7 +420,7 @@ void BatchRenamer::undoFiles( ProgressDialog* p )
 
     p->print( i18n("KRename finished the undo process."), "krename" );
     p->print( i18n("Press close to quit!") );
-    p->renamingDone( false, this, errors ); // do not allow undo from undo
+    p->renamingDone( false, false, this, errors ); // do not allow undo from undo
 
 }
 
@@ -444,7 +441,16 @@ QString BatchRenamer::processBrackets( QString text, int* length, const QString
             QString substitute = processBrackets( text.right( text.length() - pos ), &localLength, oldname, index );
             text.replace( pos - 1, localLength, substitute );
             // MSG: qDebug("substituted: %s\n", text.toUtf8().data() );
-            *length += (localLength - substitute.length() );
+            // Assure that *length does not become negative,
+            // this will cause infinite loops
+            if( localLength < substitute.length() ) 
+            {
+                *length += localLength;
+            }
+            else
+            {
+                *length += (localLength - substitute.length() );
+            }
         }
         else if( token == "]" ) 
         {
@@ -544,8 +550,8 @@ QString BatchRenamer::processString( QString text, const QString & originalName,
             int length = 0;
             QString substitute = processBrackets( text.right( text.length() - pos ), &length, oldname, index );
             text.replace( pos - 1, length, substitute );
-	    if( substitute.length() > 0 )
-		pos += substitute.length() - 1;
+            if( substitute.length() > 0 )
+                pos += substitute.length() - 1;
         }
         else if( token == "]" ) 
         {
@@ -566,7 +572,7 @@ QString BatchRenamer::processString( QString text, const QString & originalName,
             QString appendix;
             if( text[curPos] == '{' ) 
             {
-                int     appendixPos = curPos;
+                int     appendixPos = curPos + 1;
                 QString appendixToken;
                 while( (appendixPos = getNextToken( text, appendixToken, appendixPos )) != -1 )
                 {
@@ -576,7 +582,13 @@ QString BatchRenamer::processString( QString text, const QString & originalName,
                     }
                 }
 
-		// -2 because we have to go, before the current found token
+                if( appendixPos == -1 ) 
+                {
+                    // Do go into endless loop if token is not completed correctly
+                    appendixPos = text.length();
+                }
+
+                // -2 because we have to go, before the current found token
                 appendix = text.mid( curPos + 1, appendixPos - curPos - 2 );
                 appendixLength = appendixPos - curPos;
             }
@@ -584,8 +596,8 @@ QString BatchRenamer::processString( QString text, const QString & originalName,
             QString number   = processNumber( count, appendix );
             text.replace( pos - 1, (length + appendixLength), number );
 
-	    if( number.length() > 0 )
-		pos += number.length() - 1;
+            if( number.length() > 0 )
+                pos += number.length() - 1;
         }
     }
 
@@ -624,20 +636,20 @@ QString BatchRenamer::executePlugin( int index, const QString & filenameOrPath,
     QString ret = filenameOrPath;
     while( it != plugins.end() ) 
     {
-        qDebug("Checking %s", (*it)->name().toUtf8().data() );
         if( (*it)->isEnabled() && ((*it)->type() & type) )
         {
             // Every plugin should use the return value of the previous as the new filename to work on
             ret = (*it)->processFile( this, index, ret, static_cast<EPluginType>(type) );
             if( type == ePluginType_File ) 
             {
-                if( ret != QString::null  ) 
+                if( ret != QString::null ) 
                 {
                     // An error occurred -> report it
-                    p->error( ret );
+                    if( p != NULL )
+                        p->error( ret );
                     ++errorCount;
                 }
-
+                
                 ret = filenameOrPath;
             }
         }
@@ -943,7 +955,6 @@ QString BatchRenamer::findToken( const QString & oldname, QString token, int i )
             break;
     }
 
-    doEscape( token );
     return token;
 }
 
@@ -1142,58 +1153,31 @@ void BatchRenamer::writeUndoScript( QTextStream* t )
          << "fi" << endl;
 }
 
-#if 0
-void BatchRenamer::parseSubdirs( data* f )
+void BatchRenamer::createMissingSubDirs( const KRenameFile & file, ProgressDialog* p )
 {
     int pos = 0;
-    if( (pos = f->dst.name.findRev( "/", -1 ) ) > 0 ) {
-        QString dirs = f->dst.name.left( pos );
-        f->dst.name = f->dst.name.right( f->dst.name.length() - pos - 1 );
-        f->dst.directory += ( f->dst.directory.right( 1 ) == "/" ) ? "" : "/";
+    QString dstFilename = file.dstFilename();
+    if( (pos = dstFilename.lastIndexOf( '/', -1 ) ) > 0 ) {
+        QString directories = dstFilename.left( pos );
 
         // create the missing subdir now
         int i = 0;
-        QString d = "";
-        while( (d = dirs.section( "/", i, i, QString::SectionSkipEmpty )) && ! d.isEmpty() ) { // asignment here!
-            KURL url = f->dst.url;
+        QString d = directories.section( "/", i, i, QString::SectionSkipEmpty );
+        KUrl url = file.dstUrl();            
+        while( !d.isEmpty() ) {
             // it is important to unescape here
             // to support dirnames containing "&" or 
             // similar tokens
             url.addPath( unEscape( d ) );
-            if( !NetAccess::exists( url ) && !NetAccess::mkdir( url ) )
-                // TODO: GUI bug report
-                qDebug("Can't create %s", url.prettyURL().latin1() ); 
+            if( !KIO::NetAccess::exists( url, true, p ) && !KIO::NetAccess::mkdir( url, p ) ) {
+                p->error( i18n("Cannot create directory %1", url.prettyUrl()) );
+            }
 
-            f->dst.url.addPath( d );
-            f->dst.directory.append( d + "/" );
             i++;
+            d = directories.section( "/", i, i, QString::SectionSkipEmpty );
         }
     }
 }
-#endif // 0
-
-bool BatchRenamer::applyManualChanges( int i )
-{
-    /*
-     * The last step: make changes of
-     * the user visible
-     */
-
-    if( !m_changes.isEmpty() )
-        for( int z = 0; z < m_changes.count(); z++ ) {
-            KUrl file = m_changes[z].url;
-            if( file == (*m_files)[i].srcUrl() ) {
-                (*m_files)[i].setDstFilename( m_changes[z].user );
-
-                // the file extension is already included
-                // in the users name
-                (*m_files)[i].setDstExtension( QString::null );
-                return true;
-            }
-        }
-
-    return false;
-}
 
 void BatchRenamer::findCounterReset( int i )
 {
